# Copyright (c) 2020, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods, bad-continuation

"""Test cases for the magma module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import centos, docker, ubuntu

from hpccm.building_blocks.magma import magma
from hpccm.toolchain import toolchain

class Test_sensei(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """Default magma building block"""
        m = magma()
        self.assertEqual(str(m),
r'''# MAGMA version 2.5.3
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp http://icl.utk.edu/projectsfiles/magma/downloads/magma-2.5.3.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/magma-2.5.3.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/magma-2.5.3/build && cd /var/tmp/magma-2.5.3/build && cmake -DCMAKE_INSTALL_PREFIX=/usr/local/magma -DGPU_TARGET="Pascal Volta Turing" /var/tmp/magma-2.5.3 && \
    cmake --build /var/tmp/magma-2.5.3/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/magma-2.5.3/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/magma-2.5.3 /var/tmp/magma-2.5.3.tar.gz
ENV CPATH=/usr/local/magma/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/magma/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/magma/lib:$LIBRARY_PATH''')

    @centos
    @docker
    def test_defaults_centos(self):
        """Default magma building block"""
        m = magma()
        self.assertEqual(str(m),
r'''# MAGMA version 2.5.3
RUN yum install -y \
        tar \
        wget && \
    rm -rf /var/cache/yum/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp http://icl.utk.edu/projectsfiles/magma/downloads/magma-2.5.3.tar.gz && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/magma-2.5.3.tar.gz -C /var/tmp -z && \
    mkdir -p /var/tmp/magma-2.5.3/build && cd /var/tmp/magma-2.5.3/build && cmake -DCMAKE_INSTALL_PREFIX=/usr/local/magma -DGPU_TARGET="Pascal Volta Turing" /var/tmp/magma-2.5.3 && \
    cmake --build /var/tmp/magma-2.5.3/build --target all -- -j$(nproc) && \
    cmake --build /var/tmp/magma-2.5.3/build --target install -- -j$(nproc) && \
    rm -rf /var/tmp/magma-2.5.3 /var/tmp/magma-2.5.3.tar.gz
ENV CPATH=/usr/local/magma/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/magma/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/magma/lib:$LIBRARY_PATH''')

    @ubuntu
    @docker
    def test_runtime(self):
        """Runtime"""
        m = magma()
        r = m.runtime()
        self.assertEqual(r,
r'''# MAGMA
COPY --from=0 /usr/local/magma /usr/local/magma
ENV CPATH=/usr/local/magma/include:$CPATH \
    LD_LIBRARY_PATH=/usr/local/magma/lib:$LD_LIBRARY_PATH \
    LIBRARY_PATH=/usr/local/magma/lib:$LIBRARY_PATH''')



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods, bad-continuation

"""Test cases for the openmpi module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import centos, docker, ubuntu

from hpccm.building_blocks.nvhpc import nvhpc
from hpccm.building_blocks.openmpi import openmpi

class Test_openmpi(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @ubuntu
    @docker
    def test_defaults_ubuntu(self):
        """Default openmpi building block"""
        ompi = openmpi()
        self.assertEqual(str(ompi),
r'''# OpenMPI version 4.0.5
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        bzip2 \
        file \
        hwloc \
        libnuma-dev \
        make \
        openssh-client \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://www.open-mpi.org/software/ompi/v4.0/downloads/openmpi-4.0.5.tar.bz2 && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/openmpi-4.0.5.tar.bz2 -C /var/tmp -j && \
    cd /var/tmp/openmpi-4.0.5 &&   ./configure --prefix=/usr/local/openmpi --disable-getpwuid --enable-orterun-prefix-by-default --with-cuda --with-verbs && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/openmpi-4.0.5 /var/tmp/openmpi-4.0.5.tar.bz2
ENV LD_LIBRARY_PATH=/usr/local/openmpi/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/openmpi/bin:$PATH''')

    @centos
    @docker
    def test_defaults_centos(self):
        """Default openmpi building block"""
        ompi = openmpi()
        self.assertEqual(str(ompi),
r'''# OpenMPI version 4.0.5
RUN yum install -y \
        bzip2 \
        file \
        hwloc \
        make \
        numactl-devel \
        openssh-clients \
        perl \
        tar \
        wget && \
    rm -rf /var/cache/yum/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://www.open-mpi.org/software/ompi/v4.0/downloads/openmpi-4.0.5.tar.bz2 && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/openmpi-4.0.5.tar.bz2 -C /var/tmp -j && \
    cd /var/tmp/openmpi-4.0.5 &&   ./configure --prefix=/usr/local/openmpi --disable-getpwuid --enable-orterun-prefix-by-default --with-cuda --with-verbs && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/openmpi-4.0.5 /var/tmp/openmpi-4.0.5.tar.bz2
ENV LD_LIBRARY_PATH=/usr/local/openmpi/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/openmpi/bin:$PATH''')

    @ubuntu
    @docker
    def test_ldconfig(self):
        """ldconfig option"""
        ompi = openmpi(ldconfig=True, version='3.1.2')
        self.assertEqual(str(ompi),
r'''# OpenMPI version 3.1.2
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        bzip2 \
        file \
        hwloc \
        libnuma-dev \
        make \
        openssh-client \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://www.open-mpi.org/software/ompi/v3.1/downloads/openmpi-3.1.2.tar.bz2 && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/openmpi-3.1.2.tar.bz2 -C /var/tmp -j && \
    cd /var/tmp/openmpi-3.1.2 &&   ./configure --prefix=/usr/local/openmpi --disable-getpwuid --enable-orterun-prefix-by-default --with-cuda --with-verbs && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    echo "/usr/local/openmpi/lib" >> /etc/ld.so.conf.d/hpccm.conf && ldconfig && \
    rm -rf /var/tmp/openmpi-3.1.2 /var/tmp/openmpi-3.1.2.tar.bz2
ENV PATH=/usr/local/openmpi/bin:$PATH''')

    @ubuntu
    @docker
    def test_nvhpc(self):
        """HPC SDK toolchain"""
        compiler = nvhpc(eula=True)
        ompi = openmpi(toolchain=compiler.toolchain, version='4.0.4')
        self.assertEqual(str(ompi),
r'''# OpenMPI version 4.0.4
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        bzip2 \
        file \
        hwloc \
        libnuma-dev \
        make \
        openssh-client \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://www.open-mpi.org/software/ompi/v4.0/downloads/openmpi-4.0.4.tar.bz2 && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/openmpi-4.0.4.tar.bz2 -C /var/tmp -j && \
    cd /var/tmp/openmpi-4.0.4 &&  CC=nvc CFLAGS=-O1 CXX=nvc++ F77=nvfortran F90=nvfortran FC=nvfortran FCFLAGS='-fpic -DPIC' ./configure --prefix=/usr/local/openmpi --disable-getpwuid --enable-orterun-prefix-by-default --with-cuda --with-verbs && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/openmpi-4.0.4 /var/tmp/openmpi-4.0.4.tar.bz2
ENV LD_LIBRARY_PATH=/usr/local/openmpi/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/openmpi/bin:$PATH''')

    @ubuntu
    @docker
    def test_pmi(self):
        """pmi options"""
        ompi = openmpi(pmi='/usr/local/slurm-pmi2', pmix='internal',
                       version='4.0.1')
        self.assertEqual(str(ompi),
r'''# OpenMPI version 4.0.1
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        bzip2 \
        file \
        hwloc \
        libnuma-dev \
        make \
        openssh-client \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && wget -q -nc --no-check-certificate -P /var/tmp https://www.open-mpi.org/software/ompi/v4.0/downloads/openmpi-4.0.1.tar.bz2 && \
    mkdir -p /var/tmp && tar -x -f /var/tmp/openmpi-4.0.1.tar.bz2 -C /var/tmp -j && \
    cd /var/tmp/openmpi-4.0.1 &&   ./configure --prefix=/usr/local/openmpi --disable-getpwuid --enable-orterun-prefix-by-default --with-cuda --with-pmi=/usr/local/slurm-pmi2 --with-pmix=internal --with-verbs && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/openmpi-4.0.1 /var/tmp/openmpi-4.0.1.tar.bz2
ENV LD_LIBRARY_PATH=/usr/local/openmpi/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/openmpi/bin:$PATH''')

    @ubuntu
    @docker
    def test_git_repository_true(self):
        """git repository True"""
        ompi = openmpi(repository=True)
        self.assertEqual(str(ompi),
r'''# OpenMPI https://github.com/open-mpi/ompi.git
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        autoconf \
        automake \
        bzip2 \
        ca-certificates \
        file \
        git \
        hwloc \
        libnuma-dev \
        libtool \
        make \
        openssh-client \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && cd /var/tmp && git clone --depth=1 --recursive https://github.com/open-mpi/ompi.git ompi && cd - && \
    cd /var/tmp/ompi && \
    ./autogen.pl && \
    cd /var/tmp/ompi &&   ./configure --prefix=/usr/local/openmpi --disable-getpwuid --enable-orterun-prefix-by-default --with-cuda --with-verbs && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/ompi
ENV LD_LIBRARY_PATH=/usr/local/openmpi/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/openmpi/bin:$PATH''')

    @ubuntu
    @docker
    def test_git_repository_value(self):
        """git repository value"""
        ompi = openmpi(branch='v4.0.x',
                       repository='https://github.com/open-mpi-fork/ompi.git')
        self.assertEqual(str(ompi),
r'''# OpenMPI https://github.com/open-mpi-fork/ompi.git v4.0.x
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        autoconf \
        automake \
        bzip2 \
        ca-certificates \
        file \
        git \
        hwloc \
        libnuma-dev \
        libtool \
        make \
        openssh-client \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && cd /var/tmp && git clone --depth=1 --branch v4.0.x --recursive https://github.com/open-mpi-fork/ompi.git ompi && cd - && \
    cd /var/tmp/ompi && \
    ./autogen.pl && \
    cd /var/tmp/ompi &&   ./configure --prefix=/usr/local/openmpi --disable-getpwuid --enable-orterun-prefix-by-default --with-cuda --with-verbs && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/ompi
ENV LD_LIBRARY_PATH=/usr/local/openmpi/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/openmpi/bin:$PATH''')

    @ubuntu
    @docker
    def test_cuda_path(self):
        """git repository value"""
        ompi = openmpi(branch='v4.0.x',
                       repository='https://github.com/open-mpi-fork/ompi.git',
                       cuda='/usr/local/cuda2')
        self.assertEqual(str(ompi),
r'''# OpenMPI https://github.com/open-mpi-fork/ompi.git v4.0.x
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        autoconf \
        automake \
        bzip2 \
        ca-certificates \
        file \
        git \
        hwloc \
        libnuma-dev \
        libtool \
        make \
        openssh-client \
        perl \
        tar \
        wget && \
    rm -rf /var/lib/apt/lists/*
RUN mkdir -p /var/tmp && cd /var/tmp && git clone --depth=1 --branch v4.0.x --recursive https://github.com/open-mpi-fork/ompi.git ompi && cd - && \
    cd /var/tmp/ompi && \
    ./autogen.pl && \
    cd /var/tmp/ompi &&   ./configure --prefix=/usr/local/openmpi --disable-getpwuid --enable-orterun-prefix-by-default --with-cuda=/usr/local/cuda2 --with-verbs && \
    make -j$(nproc) && \
    make -j$(nproc) install && \
    rm -rf /var/tmp/ompi
ENV LD_LIBRARY_PATH=/usr/local/openmpi/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/openmpi/bin:$PATH''')

    @ubuntu
    @docker
    def test_runtime(self):
        """Runtime"""
        ompi = openmpi()
        r = ompi.runtime()
        self.assertEqual(r,
r'''# OpenMPI
RUN apt-get update -y && \
    DEBIAN_FRONTEND=noninteractive apt-get install -y --no-install-recommends \
        hwloc \
        openssh-client && \
    rm -rf /var/lib/apt/lists/*
COPY --from=0 /usr/local/openmpi /usr/local/openmpi
ENV LD_LIBRARY_PATH=/usr/local/openmpi/lib:$LD_LIBRARY_PATH \
    PATH=/usr/local/openmpi/bin:$PATH''')

    def test_toolchain(self):
        """Toolchain"""
        ompi = openmpi()
        tc = ompi.toolchain
        self.assertEqual(tc.CC, 'mpicc')
        self.assertEqual(tc.CXX, 'mpicxx')
        self.assertEqual(tc.FC, 'mpifort')
        self.assertEqual(tc.F77, 'mpif77')
        self.assertEqual(tc.F90, 'mpif90')



# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, too-few-public-methods

"""Test cases for the copy module"""

from __future__ import unicode_literals
from __future__ import print_function

import logging # pylint: disable=unused-import
import unittest

from helpers import bash, docker, invalid_ctype, singularity, singularity26, singularity32, singularity37

from hpccm.primitives.copy import copy

class Test_copy(unittest.TestCase):
    def setUp(self):
        """Disable logging output messages"""
        logging.disable(logging.ERROR)

    @docker
    def test_empty(self):
        """No source or destination specified"""
        c = copy()
        self.assertEqual(str(c), '')

    @invalid_ctype
    def test_invalid_ctype(self):
        """Invalid container type specified"""
        c = copy(src='a', dest='b')
        with self.assertRaises(RuntimeError):
            str(c)

    @docker
    def test_single_docker(self):
        """Single source file specified"""
        c = copy(src='a', dest='b')
        self.assertEqual(str(c), 'COPY a b')

    @singularity
    def test_single_singularity(self):
        """Single source file specified"""
        c = copy(src='a', dest='b')
        self.assertEqual(str(c), '%files\n    a b')

    @bash
    def test_single_bash(self):
        """Single source file specified"""
        c = copy(src='a', dest='b')
        self.assertEqual(str(c), '')

    @docker
    def test_multiple_docker(self):
        """Multiple source files specified"""
        c = copy(src=['a1', 'a2', 'a3'], dest='b')
        self.assertEqual(str(c), 'COPY a1 \\\n    a2 \\\n    a3 \\\n    b/')

    @singularity
    def test_multiple_singularity(self):
        """Multiple source files specified"""
        c = copy(src=['a1', 'a2', 'a3'], dest='b')
        self.assertEqual(str(c), '%files\n    a1 b\n    a2 b\n    a3 b')

    @docker
    def test_files_docker(self):
        """Pairs of files specified"""
        c = copy(files={'a1': 'b1', 'a2': 'b2', 'a3': 'b3'})
        self.assertEqual(str(c), 'COPY a1 b1\nCOPY a2 b2\nCOPY a3 b3')

    @singularity
    def test_files_singularity(self):
        """Pairs of files specified"""
        c = copy(files={'a1': 'b1', 'a2': 'b2', 'a3': 'b3'})
        self.assertEqual(str(c), '%files\n    a1 b1\n    a2 b2\n    a3 b3')

    @docker
    def test_from_docker(self):
        """Docker --from syntax"""
        c = copy(src='a', dest='b', _from='dev')
        self.assertEqual(str(c), 'COPY --from=dev a b')

    @singularity26
    def test_from_singularity26(self):
        """Singularity from syntax"""
        c = copy(src='a', dest='b', _from='dev')
        self.assertEqual(str(c), '%files\n    a b')

    @singularity32
    def test_from_singularity32(self):
        """Singularity from syntax"""
        c = copy(src='a', dest='b', _from='dev')
        self.assertEqual(str(c), '%files from dev\n    a b')

    @singularity
    def test_appfiles_multiple_singularity(self):
        """Multiple app-specific source files specified"""
        c = copy(src=['a1', 'a2', 'a3'], dest='b', _app='foo')
        self.assertEqual(str(c),
                         '%appfiles foo\n    a1 b\n    a2 b\n    a3 b')

    @singularity
    def test_appfiles_files_singularity(self):
        """Pairs of app-specific files specified"""
        c = copy(files={'a1': 'b1', 'a2': 'b2', 'a3': 'b3'}, _app='foo')
        self.assertEqual(str(c),
                         '%appfiles foo\n    a1 b1\n    a2 b2\n    a3 b3')

    @docker
    def test_appfiles_docker(self):
        """app-parameter is ignored in Docker"""
        c = copy(src=['a1', 'a2', 'a3'], dest='b', _app='foo')
        self.assertEqual(str(c), 'COPY a1 \\\n    a2 \\\n    a3 \\\n    b/')

    @singularity
    def test_post_file_singularity(self):
        """Move file during post"""
        c = copy(src='a', dest='/opt/a', _post=True)
        self.assertEqual(str(c),
                         '%files\n    a /\n%post\n    mv /a /opt/a')

        c = copy(src='a', dest='/opt/', _post=True)
        self.assertEqual(str(c),
                         '%files\n    a /\n%post\n    mv /a /opt/')

    @singularity
    def test_post_multiple_singularity(self):
        """move file during post"""
        c = copy(src=['a', 'b'], dest='/opt', _post=True)
        self.assertEqual(str(c),
                         '%files\n    a /\n    b /\n%post\n    mv /a /opt/a\n    mv /b /opt/b')

    @singularity
    def test_mkdir_file_singularity(self):
        """mkdir folder with setup, single file"""
        c = copy(src='a', dest='/opt/foo/a', _mkdir=True)
        self.assertEqual(str(c),
                         '%setup\n    mkdir -p ${SINGULARITY_ROOTFS}/opt/foo\n%files\n    a /opt/foo/a')

    @singularity
    def test_mkdir_files_singularity(self):
        """mkdir folder with setup, multiple files"""
        c = copy(src=['a', 'b'], dest='/opt/foo', _mkdir=True)
        self.assertEqual(str(c),
                         '%setup\n    mkdir -p ${SINGULARITY_ROOTFS}/opt/foo\n%files\n    a /opt/foo\n    b /opt/foo')

    @docker
    def test_merge_file_docker(self):
        """merge primitives"""
        c = []
        c.append(copy(src='a', dest='A'))
        c.append(copy(src='b', dest='B'))

        merged = c[0].merge(c)
        self.assertEqual(str(merged), 'COPY a A\nCOPY b B')

    @singularity
    def test_merge_file_singularity(self):
        """merge primitives"""
        c = []
        c.append(copy(src='a', dest='A'))
        c.append(copy(src='b', dest='B'))

        merged = c[0].merge(c)
        self.assertEqual(str(merged), '%files\n    a A\n    b B')

    @docker
    def test_merge_multiple_docker(self):
        """merge primitives"""
        c = []
        c.append(copy(src=['a1', 'a2'], dest='A'))
        c.append(copy(src='b', dest='B'))

        merged = c[0].merge(c)
        self.assertEqual(str(merged), 'COPY a1 A\nCOPY a2 A\nCOPY b B')

    @singularity
    def test_merge_multiple_singularity(self):
        """merge primitives"""
        c = []
        c.append(copy(src=['a1', 'a2'], dest='A'))
        c.append(copy(src='b', dest='B'))

        merged = c[0].merge(c)
        self.assertEqual(str(merged), '%files\n    a1 A\n    a2 A\n    b B')

    @docker
    def test_merge_mixed_docker(self):
        """merge primitives"""
        c = []
        c.append(copy(src='foo', dest='bar'))
        c.append(copy(src=['1', '2', '3'], dest='/infinity'))
        c.append(copy(files={'a': '/A', 'b': '/B'}))

        merged = c[0].merge(c)
        self.assertEqual(str(merged),
r'''COPY 1 /infinity
COPY 2 /infinity
COPY 3 /infinity
COPY a /A
COPY b /B
COPY foo bar''')

    @singularity
    def test_merge_mixed_singularity(self):
        """merge primitives"""
        c = []
        c.append(copy(src='foo', dest='bar'))
        c.append(copy(src=['1', '2', '3'], dest='/infinity'))
        c.append(copy(files={'a': '/A', 'b': '/B'}))

        merged = c[0].merge(c)
        self.assertEqual(str(merged),
r'''%files
    1 /infinity
    2 /infinity
    3 /infinity
    a /A
    b /B
    foo bar''')

    @docker
    def test_chown_docker(self):
        """Docker --chown syntax"""
        c = copy(_chown='alice:alice', src='foo', dest='bar')
        self.assertEqual(str(c), 'COPY --chown=alice:alice foo bar')

    @singularity
    def test_chown_singularity(self):
        """Singularity --chown syntax"""
        c = copy(_chown='alice:alice', src='foo', dest='bar')
        self.assertEqual(str(c), '%files\n    foo bar')

    @singularity37
    def test_temp_staging(self):
        """Singularity staged files through tmp"""
        c = copy(src='foo', dest='/var/tmp/foo')
        with self.assertRaises(RuntimeError):
            str(c)

    @singularity37
    def test_from_temp_staging(self):
        """Singularity files from previous stage in tmp"""
        c = copy(_from='base', src='foo', dest='/var/tmp/foo')
        self.assertEqual(str(c), '%files from base\n    foo /var/tmp/foo')
